"""
Nagios Plugin resource(s).
"""
from logging import getLogger

from influxdb import InfluxDBClient
from nagiosplugin import (
    CheckError,
    Metric,
    Resource,
)


COUNT = "count"
MEAN = "mean"
VALUES = "values"


class Measurements(Resource):
    """
    Count and mean metrics for a simple InfluxDB measurement query.
    """
    def __init__(self,
                 hostname,
                 measurement,
                 age,
                 extra_where_clauses,
                 influxdb_hostname,
                 influxdb_port,
                 influxdb_username,
                 influxdb_password,
                 influxdb_database):
        self.hostname = hostname
        self.measurement = measurement
        self.age = age
        self.extra_where_clauses = extra_where_clauses or []
        self.influxdb_hostname = influxdb_hostname
        self.influxdb_port = influxdb_port
        self.influxdb_username = influxdb_username
        self.influxdb_password = influxdb_password
        self.influxdb_database = influxdb_database
        self.logger = getLogger('nagiosplugin')

    @property
    def fields(self):
        """
        Generate the InfluxDB query fields.
        """
        return ["time", "value"]

    @property
    def clauses(self):
        return [
            "time > now() - {}".format(self.age),
            "host = '{}'".format(self.hostname)
        ] + self.extra_where_clauses

    @property
    def query(self):
        """
        Create the InfluxDB query.
        """
        return "SELECT {} FROM {} WHERE {}".format(
            ", ".join(self.fields),
            self.measurement,
            " AND ".join(self.clauses),
        )

    def get_measurements(self):
        """
        Fetch measurements from InfluxDB.
        """
        self.logger.debug("Querying InfluxDB at {}:{} with query: \"{}\"".format(
            self.influxdb_hostname,
            self.influxdb_port,
            self.query,
        ))
        try:
            client = InfluxDBClient(
                self.influxdb_hostname,
                self.influxdb_port,
                self.influxdb_username,
                self.influxdb_password,
                self.influxdb_database,
            )
            results = client.query(self.query)
            self.logger.info("Received result set: {}".format(results))
            return list(results.get_points())
        except Exception as error:
            self.logger.info("Failed to query InfluxDB: {}".format(
                error,
            ))
            raise CheckError(error)

    def probe(self):
        """
        Query InfluxDB; yield the count and mean of the measurements.
        """
        measurements = [
            {
                field: measurement[field]
                for field in self.fields
            }
            for measurement in self.get_measurements()
        ]
        values = [measurement["value"] for mesaurement in measurements]

        count = len(measurements)
        total = float(sum(values))
        mean = 0 if count == 0 else total / count

        yield Metric(COUNT, count, context=COUNT)
        yield Metric(MEAN, mean, context=MEAN)
        # null context; values are not validated individually
        yield Metric(VALUES, values, context="null")



"""
Nagios Plugin summaries.
"""
from nagiosplugin import Summary

from influxdbnagiosplugin.resources import VALUES


def is_values(result):
    return result.metric.name == VALUES


class MeasurementValuesSummary(Summary):
    """
    Customize the success result to show all measurement values.
    """
    def __init__(self, measurement):
        self.measurement = measurement

    def ok(self, results):
        result = filter(is_values, results)[0]
        return "{} is {}".format(
            self.measurement,
            result.metric.value,
        )



#!/usr/bin/env python

from setuptools import setup, find_packages

__version__ = "1.1"

setup(
    name="influxdbnagiosplugin",
    version=__version__,
    description="InfluxDB Nagios Plugin",
    author="Location Labs",
    author_email="info@locationlabs.com",
    url="http://locationlabs.com",
    packages=find_packages(exclude=["*.tests"]),
    setup_requires=[
        "nose>=1.3.7"
    ],
    install_requires=[
        "influxdb>=2.10.0",
        "nagiosplugin>=1.2.3",
    ],
    tests_require=[
        "PyHamcrest>=1.8.5",
        "mock>=1.0.1",
        "coverage>=4.0.1",
    ],
    test_suite="influxdbnagiosplugin.tests",
    entry_points={
        "console_scripts": [
            "check-measurement = influxdbnagiosplugin.main:main",
        ]
    }
)



"""
Command line entry point.
"""
from argparse import ArgumentParser
from nagiosplugin import (
    Check,
    guarded,
    ScalarContext,
)

from influxdbnagiosplugin.resources import COUNT, MEAN, Measurements
from influxdbnagiosplugin.summaries import MeasurementValuesSummary


def add_nagios_args(parser):
    """
    Specify arguments encouraged by Nagios.
    """
    parser.add_argument(
        "--verbose",
        "-v",
        action="count",
        default=0,
    )


def add_query_args(parser):
    """
    Specify arguments used to construct InfluxDB query.
    """
    parser.add_argument(
        "--hostname",
        required=True,
        help="Monitored hostname",
    )
    parser.add_argument(
        "--measurement",
        required=True,
        help="InfluxDB measurement name",
    )
    parser.add_argument(
        "--age",
        default="30s",
        help="InfluxDB measurement age",
    )
    parser.add_argument(
        "--where",
        action="append",
        help="Extra where conditions to include.",
    )


def add_influxdb_args(parser):
    """
    Add arguments to connect to InfluxDB.
    """
    parser.add_argument(
        "--influxdb-hostname",
        default="localhost",
        help="InfluxDB hostname",
    )
    parser.add_argument(
        "--influxdb-port",
        default=8086,
        help="InfluxDB port",
    )
    parser.add_argument(
        "--influxdb-username",
        default="influxdb",
        help="InfluxDB usernanme",
    )
    parser.add_argument(
        "--influxdb-password",
        default="secret",
        help="InfluxDB password",
    )
    parser.add_argument(
        "--influxdb-database",
        default="telegraf",
        help="InfluxDB database name",
    )


def add_range_args(parser):
    """
    Add arguments for measurement range validation.
    """
    parser.add_argument(
        "--count-error-range",
        default="1:",
        help="Range of measurement counts that are NOT considered an error",
    )
    parser.add_argument(
        "--count-warning-range",
        default="2:",
        help="Range of measurement counts that are NOT considered a warning",
    )
    parser.add_argument(
        "--mean-error-range",
        default="",
        help="Range of measurement means that are NOT considered an error",
    )
    parser.add_argument(
        "--mean-warning-range",
        default="",
        help="Range of measurement counts that are NOT considered a warning",
    )


def parse_args():
    """
    Parse arguments.
    """
    parser = ArgumentParser()

    add_nagios_args(parser)
    add_query_args(parser)
    add_influxdb_args(parser)
    add_range_args(parser)

    args = parser.parse_args()
    return args


@guarded
def main():
    args = parse_args()

    check = Check(
        Measurements(
            hostname=args.hostname,
            measurement=args.measurement,
            age=args.age,
            extra_where_clauses=args.where,
            influxdb_hostname=args.influxdb_hostname,
            influxdb_port=args.influxdb_port,
            influxdb_username=args.influxdb_username,
            influxdb_password=args.influxdb_password,
            influxdb_database=args.influxdb_database,
        ),
        ScalarContext(COUNT, args.count_warning_range, args.count_error_range),
        ScalarContext(MEAN, args.mean_warning_range, args.mean_error_range),
        MeasurementValuesSummary(
            measurement=args.measurement,
        )
    )
    check.main(args.verbose)
